#include <hmg.ch>

* Simulate ? and ?? command as string concatenation
#command ?  <list,...>  =>  KOUT(  <list> )
#command ?? <list,...>  =>  KKOUT( <list> )

#command EXIT <x> => exit

#xcommand DEFAULT ;                       // Give a default value to NIL 
          <v1> TO <x1> [, <vn> TO <xn> ] ;
          => ;
          IF <v1> == NIL ; <v1> := <x1> ; END                           ;
          [; IF <vn> == NIL ; <vn> := <xn> ; END ]
          
#define HL_LenX( xValue ) ;               // Length of any value formatted by HL_Any2Str() 
                           ( LEN( HL_Any2Str( xValue ) ) )   

#define ERPLineWidth 79  // Error Report Line Width

#define ExRStr0End( cStr ) ;               // Extract & Return a string from a zero-terminated String
                           IF( CHR(0) $ cStr,;
                               LEFT( cStr, AT( CHR( 0 ), cStr ) - 1 ), cStr )
#define ExRPathFFS( cFS ) ;               // Extract & Return Path from File Specification
                     IF( "\" $ cFS, LEFT(   cFS, RAT( "\", cFS ) -1 ), '' )  // w/o last "\"   
#define ExRFNExFFS( cFS ) ;               // Extract & Return File Name ( /w extention) from File Specification
                     IF( "\" $ cFS, SUBSTR( cFS, RAT( "\", cFS ) + 1 ), cFS )  

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.
                     
PROCEDURE SEHandler( ;
                     oRTError )
                     
   LOCAL cLogFilNam := "ERR" + SUBSTR( DTOS( DATE() ), 3 ) + "_" + STRTRAN( TIME(), ":","" ) + ".LOG"

   LOCAL cLTRFName ,;
         oErrorReport,;
         lSEHError,;
         cErrorMsg
         
   LOCAL hLiterals,;
         aErrMsgLns 

   LOCAL cErrrInfo,;                   // Error Information
         cSysInfoG,;                   // SysInfo\General
         cSysInfoM,;                   // SysInfo\Memory
         cSysInfoD,;                   // SysInfo\Drives
         cSysInfoP,;                   // SysInfo\Paths & Folders
         cProgInfo,;                   // Program Information
         cTablsWAs,;                   // Tables and Work Area Items
         cCurrSets,;                   // Current settings
         cMVarInfo                     // Memory variables 
   
   ERRORBLOCK( { | oRTError | EVAL( bOldErrHandler, oRTError ) } )  

   cLTRFName    := "Literals." + UPPER( LEFT( SET( _SET_LANGUAGE ), 2 ) )
   
   oErrorReport := SEH_Report():New( cLTRFName, oRTError ) // Literal file name, Error Object
   
   lSEHError := oErrorReport:IsError()    // SEH Internal error
   cErrorMsg := ''                        

   IF ! HB_ISOBJECT( oErrorReport )
      MsgStop( "! HB_ISOBJECT", "WHY ?" )      //  is possible ?   
   ELSE
   
      IF lSEHError
         cErrorMsg := oErrorReport:ErrorMsg   
         MsgStop( cErrorMsg, "SEH Error !" )
      ELSE   
      
         SET ALTERNATE TO ( cLogFilNam )
         SET ALTERNATE ON
   
         QOUT( PADC( HL_EncloseStr( oErrorReport:Messages( 1 ) ), ERPLineWidth, "*" )   )  // Report Head ( "Error Log File" ) 
         QOUT()      
         QOUT( PADC( HL_EncloseStr( DTOC( DATE() ) + "  "  + TIME(), "( " ), ERPLineWidth ) )
         QOUT()      
         QOUT( oErrorReport:ErrorInfo() )                                  // Error Information
         QOUT()      
         QOUT( oErrorReport:SysInfo( 1 ) )   // SysInfo\General
         QOUT( oErrorReport:SysInfo( 2 ) )   // SysInfo\Memory
         QOUT( oErrorReport:SysInfo( 3 ) )   // SysInfo\Drives
         QOUT( oErrorReport:SysInfo( 4 ) )   // SysInfo\Paths & Folders
         QOUT( oErrorReport:ProgInfo() )     // Program Information
         QOUT( oErrorReport:StatInfo() )     // Status ( Tables and Work Area Items ) Info
         QOUT( oErrorReport:CurrSets() )     // Current settings
         QOUT( oErrorReport:MVarList() )     // Memory variables 
   
         QOUT()      
         QOUT( PADC( HL_EncloseStr( oErrorReport:Messages( 3 ) ), ERPLineWidth, "*" )   )  //  End of Error Report
      
         SET ALTERNATE OFF
         SET ALTERNATE TO 
         
         hLiterals  := oErrorReport:hLiterals
         aErrMsgLns := hLiterals[ "ErrMesLns" ]
         
         ShowErMsg( aErrMsgLns, cLogFilNam ) // SEH2_ErrMes( aErrMsgLns )
   
         IF TYPE( "lMaintMode" ) # "U" .AND. lMaintMode
            EXECUTE FILE "NOTEPAD.EXE" PARAMETERS ( cLogFilNam )
         ENDIF   

      ENDIF   
   ENDIF
   
   QUIT
   
RETURN // SEHandler()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

PROCEDURE ShowErMsg(;
                     aErrMsgLns,;
                     cLogFilNam )

   #define  DARKRED { 128, 0, 0 }
   
   DEFINE WINDOW frmErrMes ;
      AT 229 , 455 ; 
      WIDTH 620 ;
      HEIGHT 380 ; 
      MODAL ;
      ON INIT PlayExclamation();
      BACKCOLOR DARKRED;
      NOCAPTION      
      
      ON KEY ESCAPE OF frmErrMes ACTION frmErrMes.release
   
      DEFINE LABEL lblErMes_1
         ROW    30
         COL    10
         WIDTH  600
         HEIGHT 30
         VALUE aErrMsgLns[ 1 ] // A Run-Time Error Has Occured;
         FONTNAME "Verdana"
         FONTSIZE 16
         FONTBOLD .T.
         BACKCOLOR DARKRED
         FONTCOLOR WHITE
         CENTERALIGN .T. 
      END LABEL
   
      DEFINE LABEL lblErrMes_2
         ROW    90
         COL    10
         WIDTH  600
         HEIGHT 30
         VALUE aErrMsgLns[ 2 ] // All necessary information logged
         FONTNAME "Verdana"
         FONTSIZE 12
         TOOLTIP ""
         FONTBOLD .T.
         BACKCOLOR DARKRED
         FONTCOLOR WHITE
         CENTERALIGN .T. 
      END LABEL
   
      DEFINE LABEL lblErrMes_3
         ROW    140
         COL    10
         WIDTH  600
         HEIGHT 30
         VALUE aErrMsgLns[ 3 ] // to an errors file called"
         FONTNAME "Verdana"
         FONTSIZE 12
         FONTBOLD .T.
         BACKCOLOR DARKRED
         FONTCOLOR WHITE
         CENTERALIGN .T. 
      END LABEL
   
      DEFINE LABEL lblErrMes_4
         ROW    190
         COL    10
         WIDTH  600
         HEIGHT 30
         VALUE cLogFilNam
         FONTNAME "Verdana"
         FONTSIZE 13
         FONTBOLD .T.
         BACKCOLOR DARKRED
         FONTCOLOR YELLOW // {238,233,17}
         CENTERALIGN .T. 
      END LABEL
   
      DEFINE LABEL lblErrMes_5
         ROW    250
         COL    0
         WIDTH  600
         HEIGHT 30
         VALUE aErrMsgLns[ 5 ] // * * * CONTACT PROGRAMMER IMMEDIATELY * * *
         FONTNAME "Verdana"
         FONTSIZE 16
         FONTBOLD .T.
         BACKCOLOR DARKRED
         FONTCOLOR WHITE
         CENTERALIGN .T. 
      END LABEL
   
      DEFINE BUTTON btn_OK
         ROW    320
         COL    270
         WIDTH  100
         HEIGHT 28
         CAPTION "OK"
         ACTION  frmErrMes.release
         FONTNAME "Verdana"
         FONTSIZE 9
         FONTBOLD .T.
      END BUTTON
   
   END WINDOW // frmErrMes

   frmErrMes.Center
   frmErrMes.Activate
   
RETURN // ShowErMsg()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION SEH_Report()                       // SEH Report Class 

   STATIC SEH_oClass

   IF HB_ISNIL( SEH_oClass )
   
      SEH_oClass := HBClass():New( "SEH_Rpt" )     // SEH_Rpt : ( internal ? ) name of this class
      
      SEH_oClass:AddClassData( "cLtrFile" )        // The language literal filename
      SEH_oClass:AddClassData( "nError" )          // SEH_Report() internal error code
      SEH_oClass:AddClassData( "hLiterals" )       // Hash of Literals
      SEH_oClass:AddClassData( "oRTError" )        // Run time Error object
      
      SEH_oClass:AddMethod( "New",       @SEH_New() )        // Build a new class instance      
      SEH_oClass:AddMethod( "FName",     @SEH_FName() )      // Returns the literal file name
      SEH_oClass:AddMethod( "IsError",   @SEH_IsError() )    // Returns .T. if an SEH Internal error was recorded.  
      SEH_oClass:AddMethod( "ErrorNo",   @SEH_ErrorNo() )    // Returns the last SEH Internal error code that was recorded.
      SEH_oClass:AddMethod( "ErrorMsg",  @SEH_ErrorMsg() )   // A verbose description of SEH Internal ErrorNo
      SEH_oClass:AddMethod( "IsLFile",   @SEH_IsLFile() )    // This literal file is a valid ltr. file? 
      SEH_oClass:AddMethod( "Messages",  @SEH_Messages() )   // Page Heads and some other messages
      SEH_oClass:AddMethod( "ErrorInfo", @SEH_ErrorInfo() )  // Error Information
      SEH_oClass:AddMethod( "SysInfo",   @SEH_SystemInfo() ) // System Information
      SEH_oClass:AddMethod( "ProgInfo",  @SEH_ProgInfo() )   // Program Information
      SEH_oClass:AddMethod( "StatInfo",  @SEH_StatInfo() )   // Status ( Tables and Work Area Items ) Info 
      SEH_oClass:AddMethod( "CurrSets",  @SEH_CurrSets() )   // Current SYS SETs 
      SEH_oClass:AddMethod( "MVarList",  @SEH_MVarList() )   // Memory variables 
      SEH_oClass:Create() 
      
   ENDIF

RETURN SEH_oClass:Instance()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_Messages(;
                              nMsgNo ) 

   LOCAL oSelf := QSelf()
   
   LOCAL hLiterals := oSelf:hLiterals
         
   LOCAL aMessages := hLiterals[ "PageHeads" ],;
         cRetVal
         
   IF nMsgNo > 0 .AND. nMsgNo <= LEN( aMessages )         
      cRetVal := aMessages[ nMsgNo ]
   ENDIF   

RETURN cRetVal // SEH_RepHead1()   
*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_New( ;
                         cLtrFile,;
                         oRTError )    
                         
   LOCAL oSelf := QSelf()
                    
   oSelf:cLtrFile := cLtrFile            // Save the file name
   oSelf:nError := 0
   
   IF HB_ISNIL( cLtrFile )
      oSelf:nError := 1      // Undefined language literal file
   ELSEIF ! FILE( cLtrFile )
      oSelf:nError := 2      // Defined language literal file not found
   ELSE
      IF oSelf:IsLFile()       
         IF HB_ISOBJECT( oRTError ) 
            oSelf:oRTError := oRTError
         ELSE
            oSelf:nError := 6      // Invalid Error Object
         ENDIF   
      ENDIF
   ENDIF   

RETURN oSelf // SEH_New()   
                         
*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_IsLFile() 

   LOCAL oSelf := QSelf()
   
   LOCAL cLtrFNam := oSelf:cLtrFile,;
         cLitrTxt := '',;
         lRetVal  := .F.,;
         cLFStru1 := "5, 56, 5, 4, 19, 21, 89, 12, 6, 12, 6, 13, 51, 4, 5, 5",;
         cLFStru2 ,;
         hLiterals           

   cLitrTxt := MEMOREAD( cLtrFNam )
   
   IF EMPTY( cLitrTxt )
      oSelf:nError := 3      // Empty language literal file
   ELSE
      hLiterals := HL_Text2Hash( cLitrTxt )   
      IF HB_ISHASH( hLiterals )
         cLFStru2 := HL_Arr2List( HL_HashStat( hLiterals ) )
         IF cLFStru1 == cLFStru2
            oSelf:hLiterals := hLiterals        
            lRetVal := .T.
         ELSE
            oSelf:nError := 5   // Wrong language literal file structure
         ENDIF         
      ELSE
         oSelf:nError := 4   // Wrong language literal file ( ! Hash )
      ENDIF
   ENDIF   
   
RETURN lRetVal // SEH_IsLFile()  

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_FName()               // The literal filename associated with this class instance.
   LOCAL oSelf := QSelf()
RETURN oSelf:cLtrFile

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_IsError()             // Returns .T. if an error was recorded.  
   LOCAL oSelf := QSelf()   
RETURN ( oSelf:nError # 0 )

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_ErrorNo()             // Returns the last error code that was recorded.
   LOCAL oSelf := QSelf()   
RETURN oSelf:nError

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_ErrorMsg()            // SEH Internal error messages

   LOCAL oSelf := QSelf()   
   
   LOCAL nError := oSelf:nError,;
         cErMsg := ''   
         aErMesages := { "Undefined language literal file !",;                 // 1
                         "Defined language literal file not found !",;         // 2
                         "Empty language literal file !",;                     // 3 
                         "Wrong language literal file ( ! Hash ) !",;          // 4 
                         "Wrong language literal file structure !",;           // 5
                         "Invalid Error Object" }                              // 6
                         
   IF oSelf:IsError()
      IF nError < 1      
         cErMsg := "No errors have been recorded for SEH_Report"
      ELSEIF nError > LEN( aErMesages )
         cErMsg := "An Undefined error has occured !"
      ELSE   
         cErMsg := aErMesages[ nError ]
      ENDIF      
   ENDIF
   
RETURN cErMsg // SEH_ErrorMsg()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_ErrorInfo()                       // Error Information ( Verbose )

   LOCAL oSelf := QSelf()

   LOCAL oError := oSelf:oRTError,; 
         nLevel := 4,;                
         hLiterals := oSelf:hLiterals
         
   LOCAL aPagHeads := hLiterals[ "PageHeads" ]
         
   LOCAL aDOSErrors     := hLiterals[ "DOS_Errors" ],;
         aErrSeverities := hLiterals[ "ErrSevers" ] ,; 
         aGenErrCodes   := hLiterals[ "GenericErrs" ],;
         aPrompts       := hLiterals[ "ErrorFInfo" ],;
         aValues        := { oError:Description,;
                             HL_CallSequence( nLevel + 1 ),;
                             oError:SubCode,;
                             oError:CanSubstitute,;
                             oError:CanDefault,;
                             oError:CanRetry,;
                             oError:Description,;
                             oError:Operation,;
                             oError:FileName,;
                             HL_Any2Str( oError:OSCode ) + " : " + ;     
                                         aDOSErrors[ oError:OSCode + 1 ] ,;
                             HL_Any2Str( oError:GenCode ) + " : " + ;
                                         aGenErrCodes[ oError:GenCode + 1 ] ,;
                             HL_Any2Str( oError:Severity ) + " : " + ;     
                                         aErrSeverities[ oError:Severity + 1 ] }
   
   LOCAL cErrorInfo := '',;
         aErrorInfo := HL_ArrJoin( aPrompts, aValues ),;
         cTitle  := '',;
         aHeads  := {},;
         aAligns := { 2, 0 }                 // Alignments ( right, no )
         aDelims := { SPACE( 10 ), ":" }
   
* HL_DispAnyA( aErrorInfo, "aErrorInfo" )
   
   ? @cErrorInfo, PADC( HL_EncloseStr( aPagHeads[ 7 ] ), 79,"-") // Error Information
   
*HL_DispText( cErrorInfo, "1" )

   ? @cErrorInfo, HL_Arr2Txt( aErrorInfo,;
                              cTitle,; 
                              aHeads,; 
                              aAligns,; 
                              aDelims )
                              
*HL_DispText( cErrorInfo, "2" )

   ? @cErrorInfo
   
*HL_DispText( cErrorInfo, "3" )
   
RETURN cErrorInfo // SEH_ErrorInfo()
      
*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_SystemInfo(;          // System Information
                                nSyInfGrNo )  // 1: General, 2: Memory, 3: Drives, 4: Paths & Folders

   LOCAL oSelf := QSelf()

   LOCAL hLiterals := oSelf:hLiterals
         
   LOCAL aPagHeads := hLiterals[ "PageHeads" ]
                                   
   LOCAL aRetVal   := {},;
         aDrvList  := HL_GetDriveList(),;
         cCurDrive := '',;
         cHomDrive := '',;
         c1Drive   := '',;
         cHomeCurr := '',;
         aPrompts  := {},;
         aValues   := {}
         
   LOCAL cSysInfo  := '',;
         a1SysInf  := {},; 
         aHeads1   := {},; 
         aHeads2   := {},;
         aHeads    := {},;
         aAligns   := { 2, 0 },;                  // rigth, NO
         aDelims   := { '', ":" },;
         aTitles   := hLiterals[ "SInfoTitles" ],;
         c1Title   := '',;
         c1SIText  := '',;
         cFootNote := ''
         
   IF nSyInfGrNo < 2
      ? @cSysInfo, PADC( HL_EncloseStr( aPagHeads[ 4 ] ), 79, "-" )
      ? @cSysInfo
   ENDIF nSyInfGrNo < 2
         
   DO CASE
      CASE nSyInfGrNo = 1 // General
      
         aPrompts :=  hLiterals[ "SysInfoGen" ]
         
         aValues  := { GETENV( "COMPUTERNAME" ),; 
                       GETENV( "NUMBER_OF_PROCESSORS" ),;         
                       GETENV( "PROCESSOR_ARCHITECTURE" ),;          
                       HL_OsVers(),; 
                       GETENV( "OS" ),;   
                       GETENV( "SESSIONNAME" ),;      
                       GETENV( "LOGONSERVER" ),;   
                       GETENV( "USERDOMAIN" ),;      
                       GETENV( "USERNAME" ),;      
                       GETENV( "USERPROFILE" ),;         
                       GETENV( "COMSPEC"),;
                       GETENV( "PATHEXT" ) }      
                       
         a1SysInf :=  HL_ArrJoin( aPrompts, aValues )
         
      CASE nSyInfGrNo = 2 // Memory
      
         aPrompts :=  hLiterals[ "SysInfoMem" ]

         aValues  :=  { MemoryStatus( 1 ) / 10^3,; 
                        MemoryStatus( 2 ) / 10^3,; 
                        MemoryStatus( 3 ) / 10^3,; 
                        MemoryStatus( 4 ) / 10^3,; 
                        MemoryStatus( 5 ) / 10^3,; 
                        MemoryStatus( 6 ) / 10^3 }
                     
         a1SysInf  :=  HL_ArrJoin( aPrompts, aValues )
                       
      CASE nSyInfGrNo = 3 // Drives
      
         cCurDrive := HB_CurDrive() + ":"
         cHomDrive := GETENV( "HOMEDRIVE" )
         FOR EACH c1Drive IN aDrvList
            n1Drive   := ASC( c1Drive ) - 64
            cHomeCurr := IF( c1Drive = cCurDrive, "C", ' ' ) + ;
                         IF( c1Drive = cHomDrive, "H", ' ' ) 
            AADD( a1SysInf, { c1Drive,;                 // Drive Specifier  
                             cHomeCurr,;         
                             GetVolInfo( c1Drive + "\" ),;   // Volume Label
                             NTOC( VolSerial( c1Drive + "\" ), 16 ),;    // Volume serial no
                             DiskSpace( n1Drive ) / 10^9 } )  // HB_DISK_AVAIL *) 0 Free disk space avialable to the application 
         NEXT 
         
      CASE nSyInfGrNo = 4 // Paths & Folders
      
         aPrompts  := hLiterals[ "SysInfoPnF" ]
         
         aValues   := { GETENV( "HOMEPATH" ),;     
                        GETENV( "PATH" ),;         
                        GETENV( "TEMP" ),;         
                        GETENV( "TMP" ),;          
                        GETENV( "WINDIR" ),;       
                        GETENV( "PROGRAMFILES" ),; 
                        GetDesktopFolder(),;
                        GetMyDocumentsFolder(),;
                        GetStartUpFolder(),;
                        GetSystemFolder(),;
                        GetWindowsFolder(),;
                        GetTempFolder(),;          
                        GetCurrentFolder() }       
                        
         a1SysInf :=  HL_ArrJoin( aPrompts, aValues )
         
   ENDCASE
   c1Title := aTitles[ nSyInfGrNo ] + " :" 
   ? @c1Title
   
   IF nSyInfGrNo # 3
      aAligns := { 2, 0 }                  // rigth, NO
      aHeads  := {}
      IF nSyInfGrNo # 2
         aDelims := { '', ":" }
      ELSE
         aDelims := { SPACE( 7 ), ":"  }
      ENDIF
      cFootNote := ''
   ELSE
      aAligns := { 0, 1, 1, 1, 2 }          // no, left, left, left, rigth          
      aHeads1 := { 'D#', 'HC', 'V.Label', 'V.Serial', 'Free Sp.(GB)' }
      aHeads2 := {}
      AEVAL( aHeads1, { | c1 | AADD( aHeads2, REPLICATE( "-", LEN( c1 ) ) ) } )
      aHeads  := { aHeads1, aHeads2 }       
      aDelims := { SPACE( 10 ) }
      cFootNote := SPACE( 16 ) + aPagHeads[ 19 ] // 'H/C : H: Home Drive, C: Current Drive'
   ENDIF         
   
   c1SIText := HL_Arr2Txt( a1SysInf,;
                           c1Title,; 
                           aHeads,; 
                           aAligns,; 
                           aDelims  )
   
   ? @cSysInfo, c1SIText
   
   IF ! EMPTY( cFootNote )
      ? @cSysInfo
      ? @cSysInfo, cFootNote
      ? @cSysInfo
   ENDIF           
   
   IF nSyInfGrNo < 4
      ? @cSysInfo
      ? @cSysInfo, REPLICATE( "", 79 )
   ENDIF nSyInfGrNo < 4
                       
RETURN cSysInfo // SEH_SystemInfo()
   
*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_ProgInfo()                    // Program Info 

   LOCAL oSelf := QSelf()

   LOCAL hLiterals := oSelf:hLiterals
         
   LOCAL aPagHeads := hLiterals[ "PageHeads" ]

   LOCAL cPrgInfo := '',;
         cTitle   := '',;                // aPagHeads[ 6 ]
         aPrgInfo := {}
         
   LOCAL cPrgOrig := GetProgramFileName(),;   // Program origin (full file name)
         cPrgFold := '',;                     // Program file folder name 
         aPrompts := hLiterals[ "PrgFInfo" ],;
         aValues  := {},;
         aRetVal
        
   cPrgFold := ExRPathFFS( cPrgOrig )
   cPrgFNam := ExRFNExFFS( cPrgOrig )    
         
   aValues  := { cPrgFold,;                        
                 cPrgFNam,;                        
                 HL_Any2Str(  FILEDATE( cPrgOrig ) ),;
                 HL_Any2Str(  FILETIME( cPrgOrig ) ),;
                 HL_Any2Str(  FILESIZE( cPrgOrig ) ) }                       
                       
   aPrgInfo :=  HL_ArrJoin( aPrompts, aValues )
          
   ? @cTitle, aPagHeads[ 6 ]  // Program File :
   ? @cTitle
   
   ? @cPrgInfo, PADC( HL_EncloseStr( aPagHeads[ 5 ] ), 79, "-" ) // Program Information
   ? @cPrgInfo
   
   ? @cPrgInfo, HL_Arr2Txt( aPrgInfo,;
                               cTitle,;
                               {},;                     // Heads 
                               { 2, 0 },;               // Aligns  
                               { SPACE( 10 ), ":" } ) // Delims
                           
RETURN cPrgInfo // SEH_ProgInfo()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_StatInfo()                       // Tables in use & Work Areas info ( Verbose )


   LOCAL oSelf := QSelf()

   LOCAL hLiterals := oSelf:hLiterals
         
   LOCAL aPagHeads := hLiterals[ "PageHeads" ],;
         aPrompts  := hLiterals[ "WAsStatus" ],;
         aGPrompts := hLiterals[ "GenPrompts" ]

   LOCAL aValues    := {},;
         aRetVal    := {}

   LOCAL nCurrWANo   := SELECT(),;
         nWorkAreaNo :=  0,;
         cTablFFNam  := '',;   // Table full file name
         cTablFoldr  := '',;   // Table folder
         cTableName  := '',;   // Table folder
         nUsedWACo   :=  0     // Used Work area count
   
   LOCAL cWAStatus := '',;
         aWAStatus := {}   

   LOCAL aStruct := {},;
         cTitle  := '',;
         aHeads1 := {},;
         aHeads2 := {},;
         aHeads  := {},;       
         aAligns := {},;        
         aDelims := {}
         
         
   ? @cWAStatus, PADC( HL_EncloseStr( aPagHeads[ 8 ] ), 79, "-" )
   ? @cWAStatus
   
   FOR nWorkAreaNo := 1 TO 64
   
      DBSELECTAREA( nWorkAreaNo ) 
   
      IF !EMPTY( ALIAS( nWorkAreaNo ) )
      
         IF ++nUsedWACo > 1
            ? @cWAStatus
            ? @cWAStatus, REPLICATE( "", 79 )
            ? @cWAStatus
         ENDIF   
                     
         cTablFFNam := DBINFO( 10 )
         cTablFoldr := ExRPathFFS( cTablFFNam )
         cTablFoldr := HL_QualifyPath( cTablFoldr )
         cTableName := ExRFNExFFS( cTablFFNam )   
         
         aValues := { HB_NTOS( nWorkAreaNo ) + ;
                              IF( nCurrWANo # nWorkAreaNo, "",;
                              HL_EncloseStr( aGPrompts[ 5 ], " ( " ) ),; // " ( Current )"
                     RDDNAME()              ,;
                     DBINFO( 9 )            ,;
                     ORDBAGEXT()            ,;
                     cTablFoldr             ,; 
                     cTableName             ,; 
                     FILEDATE( cTablFFNam ) ,;
                     FILETIME( cTablFFNam ) ,;
                     FILESIZE( cTablFFNam ) ,;                       
                     ALIAS()                ,; 
                     LUPDATE()              ,; 
                     HEADER()               ,;
                     FCOUNT()               ,;
                     RECSIZE()              ,;
                     LASTREC()              ,;
                     RECNO()                ,;
                     INDEXORD()             ,;
                     DBFILTER()             ,;
                     DBRELATION()           ,; 
                     INDEXORD()             ,;
                     INDEXKEY( INDEXORD() )  }
                     
         aWAStatus := HL_ArrJoin( aPrompts, aValues )
   
         cTitle  := ''
         aHeads  := {}
         aAligns := { 2, 0 }                 // Alignments ( right, no )
         aDelims := { SPACE( 10 ), ":" }
         
         ? @cWAStatus, HL_Arr2Txt( aWAStatus,;
                                     cTitle,; 
                                     aHeads,; 
                                     aAligns,; 
                                     aDelims )
         ? @cWAStatus
                                  
         * Structure and Record data info :
         
         aStruct := DBSTRUCT()
         AEVAL( aStruct, { | a1, i1 | AADD( a1, FIELDGET( i1 ) ) } )
         ? @cTitle, SPACE( 9 ), aPagHeads[ 9 ]  + ; // Current record :
                                IF( RECNO() > LASTREC(),; 
                                HL_EncloseStr( aGPrompts[ 6 ], " ( " ), "" ) //  ( Phantom Record )

         ? @cTitle
         aHeads1 := hLiterals[ "FldListTitles" ]
         aHeads2 := {}
         AEVAL( aHeads1, { | c1 | AADD( aHeads2, REPLICATE( "-", LEN( c1 ) ) ) } )
         aHeads  := { aHeads1, aHeads2 }       
         aAligns := { 1, 3, 2, 2, 0 }         // left, cent, rigth, rigth, NO_align
         aDelims := { SPACE( 10 ) }
         
         ? @cWAStatus, HL_Arr2Txt( aStruct,;
                                     cTitle,; 
                                     aHeads,; 
                                     aAligns,; 
                                     aDelims )
         ? @cWAStatus
         
      ENDIF !EMPTY( ALIAS( nWorkAreaNo ) )
      
   NEXT 
   
   IF EMPTY( nUsedWACo )
      ? @cWAStatus, PADC( aPagHeads[ 10 ], 79, "-" ) // No work area in use.   
   ENDIF   
   
   DBSELECTAREA( nCurrWANo )
      
RETURN cWAStatus // SEH_StatInfo()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_CurrSets()                  // Current Settings ( Verbose )
   
   LOCAL oSelf := QSelf()

   LOCAL hLiterals := oSelf:hLiterals
         
   LOCAL aPagHeads := hLiterals[ "PageHeads" ],;
         aPrompts  := hLiterals[ "SYS_SETs" ]
         
   LOCAL aValues   := {  SET( _SET_EXACT         ),;  //   1
                         SET( _SET_FIXED         ),;  //   2
                         SET( _SET_DECIMALS      ),;  //   3
                         SET( _SET_DATEFORMAT    ),;  //   4
                         SET( _SET_EPOCH         ),;  //   5
                         SET( _SET_PATH          ),;  //   6
                         SET( _SET_DEFAULT       ),;  //   7
                         SET( _SET_EXCLUSIVE     ),;  //   8
                         SET( _SET_SOFTSEEK      ),;  //   9
                         SET( _SET_UNIQUE        ),;  //  10
                         SET( _SET_DELETED       ),;  //  11
                         SET( _SET_CANCEL        ),;  //  12
                         SET( _SET_DEBUG         ),;  //  13
                         SET( _SET_TYPEAHEAD     ),;  //  14
                         SET( _SET_COLOR         ),;  //  15
                         SET( _SET_CURSOR        ),;  //  16
                         SET( _SET_CONSOLE       ),;  //  17
                         SET( _SET_ALTERNATE     ),;  //  18
                         SET( _SET_ALTFILE       ),;  //  19
                         SET( _SET_DEVICE        ),;  //  20
                         SET( _SET_EXTRA         ),;  //  21
                         SET( _SET_EXTRAFILE     ),;  //  22
                         SET( _SET_PRINTER       ),;  //  23
                         SET( _SET_PRINTFILE     ),;  //  24
                         SET( _SET_MARGIN        ),;  //  25
                         SET( _SET_BELL          ),;  //  26
                         SET( _SET_CONFIRM       ),;  //  27
                         SET( _SET_ESCAPE        ),;  //  28
                         SET( _SET_INSERT        ),;  //  29
                         SET( _SET_EXIT          ),;  //  30
                         SET( _SET_INTENSITY     ),;  //  31
                         SET( _SET_SCOREBOARD    ),;  //  32
                         SET( _SET_DELIMITERS    ),;  //  33
                         SET( _SET_DELIMCHARS    ),;  //  34
                         SET( _SET_WRAP          ),;  //  35
                         SET( _SET_MESSAGE       ),;  //  36
                         SET( _SET_MCENTER       ),;  //  37
                         SET( _SET_SCROLLBREAK   ),;  //  38
                         SET( _SET_LANGUAGE      ),;  //  39
                         SET( _SET_IDLEREPEAT    ),;  //  40
                         SET( _SET_FILECASE      ),;  //  41
                         SET( _SET_DIRCASE       ),;  //  42
                         SET( _SET_DIRSEPARATOR  ),;  //  43
                         SET( _SET_EOF           ),;  //  44
                         SET( _SET_HARDCOMMIT    ),;  //  45
                         SET( _SET_FORCEOPT      ),;  //  46
                         SET( _SET_DBFLOCKSCHEME ),;  //  47
                         SET( _SET_DEFEXTENSIONS ),;  //  48
                         SET( _SET_EOL           ),;  //  49
                         SET( _SET_TRIMFILENAME  ),;  //  50
                         SET( _SET_HBOUTLOG      ),;  //  51
                         SET( _SET_HBOUTLOGINFO  ),;  //  52
                         SET( _SET_CODEPAGE      ),;  //  53
                         SET( _SET_OSCODEPAGE    ),;  //  54
                         SET( _SET_TIMEFORMAT    ),;  //  55
                         SET( _SET_DBCODEPAGE    ) }  //  56

   LOCAL cTitle    := '',;
         cRetVal   := '',;
         aCurrSets := HL_ArrJoin( aPrompts, aValues )

   ? @cTitle, PADC( HL_EncloseStr( aPagHeads[ 11 ] ), 79, "-" )
   ? @cTitle
   ? @cRetVal, HL_Arr2Txt( aCurrSets,;                // Array to convert
                          cTitle,;                   // Title
                          {},;                       // Heads
                          { 2, 0 },;                 // Alignments ( right, no )
                          { SPACE( 10 ), ":" } )     // Delimiters
                            
RETURN cRetVal // SEH_CurrSets()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION SEH_MVarList()                   // Memory Variables Info

   LOCAL oSelf := QSelf()

   LOCAL hLiterals := oSelf:hLiterals
         
   LOCAL aPagHeads := hLiterals[ "PageHeads" ]
         
   LOCAL cMemFNam := "MEMVARS.MEM",;   
         aMemVars := {},;
         cMF_Info := '',;
         aHeads1  := hLiterals[ "MemVarTitles" ],; //  { "Variable Name", "Type", "Width", "Dec", "Value" },;
         aHeads2  := {},;
         aAligns  := { 1, 3, 2, 2, 0 },;         // left, cent, rigth, rigth, NO_align
         aDelims  := { SPACE( 10 ) }

   ? @cMF_Info, PADC( HL_EncloseStr( aPagHeads[ 12 ] ), 79, "-" )
   ? @cMF_Info
   ? @cMF_Info, PADC( HL_EncloseStr( aPagHeads[ 13 ], "( " ), 79 )
   ? @cMF_Info
         
   SAVE ALL EXCEPT _* TO ( cMemFNam ) // MEMVARS
   
   aMemVars := HL_ReadMemFile( cMemFNam )
         
   AEVAL( aHeads1, { | c1 | AADD( aHeads2, REPLICATE( "-", LEN( c1 ) ) ) } )
   
   aHeads  := { aHeads1, aHeads2 }       
         
   ? @cMF_Info, HL_Arr2Txt( aMemVars,;
                               '',;           // cTitle
                               aHeads,; 
                               aAligns,; 
                               aDelims )
   ? @cMF_Info
   
   FERASE( ( cMemFNam )  ) // For inspect or use by any way .mem file, rem this line
                                
RETURN cMF_Info // SEH_MVarList()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

STATIC FUNCTION HL_ReadMemFile(;                     // Build an array with variables recorded as a .mem file
                     cMemFNam )
                     
   LOCAL aRetVal  := {},;
         cMFilStr := '',;
         c1VHeadr := '',;
         c1VName  := '',;
         c1VType  := '',;
         c1VData  := '',;
         x1VValu  := '',;
         n1VLeng  := 0,;
         n1VDeci  := 0,;
         n1VWidt  := 0,;
         nFilPntr := 1,;
         nMFStrLn := 0,;
         nActLen  := 0   // Actual Length ( regarding value )
   
   cMFilStr := MEMOREAD( cMemFNam )
   
   nMFStrLn := LEN ( cMFilStr )
   
   WHILE nFilPntr < nMFStrLn 
    
      c1VHeadr := SUBSTR( cMFilStr, nFilPntr, 20 )
      c1VName  := ExRStr0End( LEFT( c1VHeadr, 11 ) )
      c1VType  := CHR( ASC(SUBSTR( c1VHeadr, 12, 1 ) ) - 128 )      
      n1VDeci  := ASC( SUBSTR( c1VHeadr, 18, 1 ) )
      
      nFilPntr += 32      
      
      DO CASE 
         CASE c1VType = "L"
              n1VWidt := 1
              c1VData := SUBSTR( cMFilStr, nFilPntr, n1VWidt )
              x1VValu := ASC( c1VData ) > 0
              nActLen := 1
         CASE c1VType = "D"
              n1VWidt := 8
              c1VData := SUBSTR( cMFilStr, nFilPntr, n1VWidt )
              x1VValu := ( CTOF( c1VData ) - 2415021 ) + CTOD( "01/01/1900" ) 
              nActLen := 8
         CASE c1VType = "N"
              n1VWidt := 8
              c1VData := SUBSTR( cMFilStr, nFilPntr, n1VWidt )
              x1VValu := HL_Any2Str( CTOF( c1VData ) )
              nActLen := LEN( x1VValu )              
         OTHERWISE // "C"  
              n1VLeng  := ASC( SUBSTR( c1VHeadr, 17, 1 ) )
              n1VWidt  := n1VDeci * 256 + n1VLeng - 1
              c1VData := SUBSTR( cMFilStr, nFilPntr, n1VWidt )                        
              x1VValu := ExRStr0End( c1VData )             
              nActLen := LEN( x1VValu )                            
      ENDCASE
      
      AADD( aRetVal, { c1VName,;    // Variable name
                       c1VType,;    // Variable type 
                       nActLen,;    // Actual lenght of variable
                       n1VDeci,;    // Decimal digit count
                       x1VValu } )  // Recorded value of variable
      
      nFilPntr += n1VWidt + IF( c1VType = "C", 1, 0 )
      
   ENDDO   
   
RETURN aRetVal // HL_ReadMemFile()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION HL_Text2Hash(;            // Convert a text (List) to Hash 
                     cHashStr )              // for only a hash with predifined structure  ( <cHKey> => <aArray> )   

   LOCAL c1Line   := '',;
         c1Item   := '',;
         hRetVal  := { => },;
         cHKey ,;
         aHVal 
   
   LOCAL aHomeArr := HB_ATOKENS( cHashStr, CRLF )
   
   nLineCo := 0
   FOR EACH c1Line IN aHomeArr
      IF ! EMPTY( c1Line )
         ++nLineCo
         c1Item := c1Line  // TRIM( c1Line )
         IF ! EMPTY( LEFT( c1Line, 1 ) ) // LEN( c1Item ) = LEN( c1Line )
            IF !EMPTY( aHVal )
               hRetVal[ cHKey ] := aHVal
            ENDIF
            cHKey := c1Item
            aHVal := {}
         ELSE
            AADD( aHVal, ALLTRIM( c1Item ) )
         ENDIF            
      ENDIF   
   NEXT c1Line
   
   hRetVal[ cHKey ] := aHVal  // Last Item
   
RETURN hRetVal // HL_Text2Hash()   

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION HL_HashStat(;             // Statistics ( items and data count ) of a Hash
                             hHash )         // for any hash, only first level; no nesting. 
                
   LOCAL aRetVal := {},;
         a1HItem ,;
         xHVal 
  
   FOR EACH a1HItem IN hHash
      xHVal := a1HItem:__enumValue()        
      AADD( aRetVal, IF( HB_ISARRAY( xHVal ) .OR. HB_ISHASH( xHVal ), LEN( xHVal ), 1 ) )
   NEXT

RETURN aRetVal // HL_HashStat()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION HL_Arr2List(;                   // (1 dim ) Array -> List 
                              aArray )
                       
   LOCAL cRVal := ''
   
   IF HB_ISARRAY( aArray )
      AEVAL( aArray, { | x1, i1 | cRVal += HL_Any2Str( x1 ) + IF( i1 < LEN( aArray ), ', ', '') } )      
   ENDIF   
      
   RETURN cRVal // HL_Arr2List()                           

*.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

FUNCTION HL_EncloseStr( ;
                                 cStrg,;   // String to enclose
                                 cLEnc,;   // Left Enclosing string ( default : " " )
                                 cREnc )   // Right Enclosing string ( default : " " )
                                 
   LOCAL cTmlat := "  ()<>[]{}" + [''""],;
         nPLEnc :=  0,;
         c1LEnc := '',;
         nPCPos := 0
   
   IF HB_ISNIL( cLEnc )
      cLEnc := " "
   ENDIF
   
   IF HB_ISNIL( cREnc )
      cREnc := ''
      FOR nPLEnc := 1 TO LEN( cLEnc )
         c1LEnc := SUBSTR( cLEnc, nPLEnc, 1 )
         nPCPos := AT( c1LEnc, cTmlat )         
         cREnc  := IF( nPCPos > 0, SUBSTR( cTmlat, nPCPos + 1, 1 ), c1LEnc ) + cREnc
      NEXT 
   ENDIF HB_ISNIL( cREnc )

RETURN ( cLEnc + cStrg + cREnc )  // HL_EncloseStr()
  
*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION HL_CallSequence( nBegLevel )

   LOCA nLevel  := nBegLevel,;
        aRetVal := {}

   WHILE !(PROCNAME( nLevel ) == "")
      AADD( aRetVal, PROCNAME( nLevel ) + ;
                      " (" + HB_NTOS( PROCLINE( nLevel ) ) + ")" )
      nLevel++                    
   ENDDO
   
RETURN aRetVal // HL_CallSequence()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION HL_Any2Str( ;                       // Convert any type data to String.
                     xAny )

   LOCAL cRVal  := '???',;
         nType  :=  0,;
         aCases := { { "A", { | x | "{...}" } },;                
                     { "B", { | x | "{||}" } },;                
                     { "C", { | x | IF( LEN( x ) < 3 .AND. HL__ISINCCTRL( x ), "0x" + HL__STRG2HEX( x ), x ) }},;
                     { "H", { | x | "<Hash>" } },;
                     { "M", { | x | x   } },;                   
                     { "D", { | x | DTOC( x ) } },;             
                     { "L", { | x | IF( x,".T.",".F.") } },;    
                     { "N", { | x | LTRIM( TRAN( x, IF( INT( x ) # x,;
                                                    "999,999,999,999,999,999.999",;
                                                    IF( LEN( LTRIM( STR( x ) ) ) > 4,;                                                      
                                                    "999,999,999,999,999,999,999",;
                                                    "9999" ) ) ) ) } },;
                     { "O", { | x | ":Object:" } },;
                     { "U", { | x | "<NIL>" } } }
      

   IF (nType := ASCAN( aCases, { | a1 | VALTYPE( xAny ) == a1[ 1 ] } ) ) > 0
      cRVal := EVAL( aCases[ nType, 2 ], xAny )
   ENDIF
   
RETURN cRVal // HL_Any2Str()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION HL_ArrJoin( ;
                     aArray1,; 
                     aArray2 )
                     
   LOCAL aRetVal := {}
   
   AEVAL( aArray1, { | x1, i1 | AADD( aRetVal, { x1, IF( i1 <= LEN( aArray2 ),;
                                                 aArray2[ i1 ], ) } ) } )
   
RETURN aRetVal // HL_ArrJoin()
  
*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._
  
PROCEDURE KOUT( ... )                     // Concatanate expressions as a string, with CRLF at end

   LOCAL aParams := HB_APARAMS()

   IF PCOUNT() > 0
      cHomeString := aParams[ 1 ]  // MUST NOT be any Variable-Name conflict
      cHomeString += CRLF
      KKOUT( ... )
   ENDIF PCOUNT() > 0

RETURN // KOUT()   

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

PROCEDURE KKOUT( ... )                    // Concatanate expressions as a string, without CRLF at end

   LOCAL nPCount := PCOUNT(),;
         aParams := HB_APARAMS(),;
         nParNo  := 0
         
   IF nPCount > 0
      cHomeString := aParams[ 1 ] // MUST NOT be any Variable-Name conflict
      IF LEN( aParams ) > 1
         FOR nParNo := 2 TO LEN( aParams )
            cHomeString += HL_Any2Str( aParams[ nParNo ] ) + " " 
         NEXT
      ENDIF   
   ENDIF nPCount > 0
   
RETURN // KKOUT()   

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

FUNCTION HL_QualifyPath(;                    // Convert ( if necessary ) relative path to absolute 
                  cPath )   
                  
   LOCAL cCurrFolder := GetCurrentFolder()                  
         cRetVal := cPath
   
   IF SUBSTR( cPath, 2, 1 ) # ":"   
      SetCurrentFolder( cPath )
      cRetVal := GetCurrentFolder()
      SetCurrentFolder( cCurrFolder )                             
   ENDIF
   
RETURN cRetVal // HL_QualifyPath()
   
*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

FUNCTION HL_Arr2Txt( ;                 // Convert any MAX THREE dim array to text
                        aArray,;
                        cTitle,; 
                        aHeads,; 
                        aAligns,; 
                        aDelims )
                        
   LOCAL cRetVal  := '',;
         aMaxLens := {},;         
         c1Line   := '',;
         x1Line   := '',;
         n1Align  :=  0,;         
         c1Cell   := '',;  
         nMax1Len :=  0,;
         c1Value  := '',;   
         c1Delim  := '',;
         cHead1Ln := '',;
         cHead2Ln := '',;
         aCellArr := {},;
         nCArrPos :=  0,;         
         nCAr1Len :=  0,;         
         x1Value  
         
   DEFAULT cTitle  TO '',;
           aDelims TO {},;
           aAligns TO {},; 
           aDelims TO {}
           
   cRetVal := cTitle
   
   IF HB_ISARRAY( aHeads ) .AND. ! EMPTY( aHeads )
      IF HB_ISARRAY( aHeads[ 1 ] )
         ASIZE( aArray, LEN( aArray ) + 1 )
         AINS( aArray, 1 )
         aArray[ 1 ] := aHeads[ 1 ] 
      ENDIF
      IF HB_ISARRAY( aHeads[ 2 ] ) .AND. LEN( aHeads ) > 1     
         ASIZE( aArray, LEN( aArray ) + 1 )
         AINS( aArray, 2 )   
         aArray[ 2 ] := aHeads[ 2 ] 
      ENDIF   
   ENDIF
   
   aMaxLens := HL_ArrMaxLens( aArray )
   
   FOR EACH x1Line IN aArray
   
      IF HB_ISARRAY( x1Line )
      
         c1Line := ''
         
         FOR nRowNo := 1 TO LEN( x1Line )
         
            x1Value  := x1Line[ nRowNo ]
            
            IF HB_ISARRAY( x1Value ) // cell is an array            
            
               nCArrPos := LEN( c1Line )          
               
               nCAr1Len := aMaxLens[ nRowNo ] 
            
               AEVAL( x1Value, { | x1, i1 | AADD( aCellArr, x1 ) } )
               
               x1Value := x1Value[ 1 ] 
               
            ENDIF
            
            nMax1Len := aMaxLens[ nRowNo ]
            
            c1Value  := HL_Any2Str( x1Value )
            
            IF LEN( aAligns ) < nRowNo
               ASIZE( aAligns, nRowNo )
            ENDIF
            
            IF LEN( aDelims  ) < nRowNo
               ASIZE( aDelims, nRowNo )
            ENDIF
            
            DEFAULT aAligns[ nRowNo ] TO 2,;    // 2: Right aling, default
                    aDelims[ nRowNo ] TO " "    // " ", default ( including 1st row ! )
                    
            n1Align := aAligns[ nRowNo ]
            c1Delim := aDelims[ nRowNo ]

            c1Cell := IF(n1Align > 2, PADC( c1Value, nMax1Len ),;
                      IF(n1Align > 1, PADL( c1Value, nMax1Len ),;
                      IF(n1Align > 0, PADR( c1Value, nMax1Len ),;
                                            c1Value ) ) )   
                                      
            ?? @c1Line, c1Delim, c1Cell
            
         NEXT nRowNo 

         ? @cRetVal, c1Line
         
         IF nCArrPos > 0         // "A" type row value
            FOR nCAr1ElmNo := 2 TO LEN( aCellArr )
               c1Value := HL_Any2Str( aCellArr[ nCAr1ElmNo ] )
               ? @cRetVal, SPACE( nCArrPos + 1 ), c1Value
            NEXT            
            nCArrPos := 0      
         ENDIF
         
      ELSE
        * Single dim        
        ? @cRetVal, x1Line
      ENDIF
      
   NEXT x1Line

RETURN cRetVal // HL_Arr2Txt()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

FUNCTION HL_OsVers()

   LOCAL aWinVers := WindowsVersion() 
   
   LOCAL cRetVal := TRIM( aWinVers[ 1 ] ) + ", " + aWinVers[ 3 ]
   
RETURN cRetVal // HL_OsVers()  

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.


FUNCTION HL__STRG2HEX( c1 )                    // Convert string to hex
RETURN ( HL__STRGEVAL( c1, '',, { | c1, i1, cTrg | cTrg += NTOC( ASC( c1 ), 16, 2, "0" ) } ) )

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

FUNCTION HL__ISINCCTRL( ;                       // Is this string include any control character ( ASC < 32 ) ?
                       cString )
   LOCAL lRVal := .F.,;
         nCPos := 0
         
   IF HB_ISCHAR( cString )
      FOR nCPos := 1 TO LEN( cString )
         IF ASC( SUBSTR( cString, nCPos, 1 ) ) < 32
            lRVal := .T.
            EXIT FOR
         ENDIF    
      NEXT nCPos 
   ENDIF
   
RETURN lRVal // HL__ISINCCTRL()

*.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._
                       
FUNCTION HL__STRGEVAL(;                              // String Evaluation (AEVAL'n Katar'cas) (eski ad SEVAL idi)
                     cSource,;  // Source (String)
                     xRVal ,;   // Return Value ( any type )
                     nStep ,;
                     bBlock )   // Code Block to evaluate

   LOCAL nSInd  :=  0,;
        cSubS := ''        // sub string 

   DEFAULT nStep TO 1
   
   FOR nSInd := 1 TO LEN( cSource ) STEP nStep
      cSubS := SUBSTR( cSource, nSInd, nStep )
      EVAL( bBlock, cSubS, nSInd, @xRVal )
   NEXT nSInd

RETURN xRVal // HL__STRGEVAL()

*`*..*`*..*`*..*`*..*`*..*`*..*`*..*`*..*`*..*`*..*`*.

FUNCTION HL_ArrMaxLens( ;                  // Length of longest elementS in an array
                       aArray ) 
                       
   LOCAL nRetVal  :=  0,;
         aRetVal  := {},;        
         nCurLen  :=  0,;
         nRowNo   :=  0,;
         x1D1Elem ,;
         x1D2Elem ,;
         xRetVal
         
   FOR EACH x1D1Elem IN aArray
   
      IF HB_ISARRAY( x1D1Elem ) 
         FOR nRowNo := 1 TO LEN( x1D1Elem )
            x1D2Elem := x1D1Elem[ nRowNo ]
            IF HB_ISARRAY( x1D2Elem ) 
            ELSE
               nCurLen := HL_LenX( x1D2Elem )
            ENDIF
            IF LEN( aRetVal ) < nRowNo             
               AADD( aRetVal, 0 )
            ENDIF   
            aRetVal[ nRowNo ] := MAX( nCurLen, aRetVal[ nRowNo ] )
         NEXT x1D2Elem
      ELSE
         * All non array x1D1Elem ( row / line ) are single elem 
         nCurLen := HL_LenX( x1D1Elem )
         IF EMPTY( aRetVal )
            AADD( aRetVal, 0 )
         ENDIF   
         aRetVal[ 1 ] := MAX( nCurLen, aRetVal[ 1 ] )
      ENDIF
      
   NEXT x1D1Elem   

   IF LEN( aRetVal ) = 1 .AND. HB_ISNUMERIC( aRetVal[ 1 ] )
      xRetVal := aRetVal[ 1 ]
   ELSE   
      xRetVal := aRetVal
   ENDIF
   
RETURN XRetVal // HL_ArrMaxLens()

*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.

/*

   f.HL_GetDriveList() : Produce a list of ready drives in system.
   
   Syntax :  HL_GetDriveList() --> aDrives
   
   Argument : No argument required.
   
   Return   : aDrives : An array have ready drive letter.
              Although exist, not-ready drives not included. 
   
   History  : 9.2006 : First Release
   
   Copyright 2006 Bicahi Esgici <esgici@gmail.com>
   
*/

FUNC HL_GetDriveList()

   LOCA aRVal := {},;
        nDriv :=  0,;
        cDriv := "",;
        cCurDir := GetCurrentFolder()
        
   FOR nDriv := 0 TO 28
      cDriv := CHR( 67 + nDriv ) + ":\"
      IF DIRCHANGE( cDriv ) < 1
         AADD( aRVal, LEFT( cDriv, 2 ) )
      ENDIF
   NEXT nDriv
   
   SetCurrentFolder( cCurDir )     
   
RETU aRVal // HL_GetDriveList()
           
*-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._.-._

